// useful to have them as global variables
var canvas, ctx, w, h; 
var mousePos;

// an empty array!
var balls = []; 
var initialNumberOfBalls;
var globalSpeedMutiplier = 1;
var colorToEat = 'red';
var wrongBallsEaten = goodBallsEaten = 0;
var numberOfGoodBalls;
var level = 1;

// game state (game over, game running, starting screen, hi score screen, options  etc.)
let gameState = "StartMenu";
let currentScore = 0;

var player = {
  x:10,
  y:10,
  width:20,
  height:20,
  color:'red'
}

let numberOfLives = 5;

window.onload = function init() {
    // called AFTER the page has been loaded
    canvas = document.querySelector("#myCanvas");
  
    // often useful
    w = canvas.width; 
    h = canvas.height;  
  
    // important, we will draw with this object
    ctx = canvas.getContext('2d');
  
    // add a mousemove event listener to the canvas
    canvas.addEventListener('mousemove', mouseMoved);
    document.addEventListener('keydown', keyPressed);


    // ready to go !
    mainLoop();
};

function keyPressed(event) {
  console.log('key pressed : ' + event.key);
  if(event.key === " ") {
    if(gameState === "GameOverMenu") {
      // reset the level to 0, the score to 0 etc.
      restartGame();
    } else if(gameState === "StartMenu") {
      gameState = "GameRunning";
      startGame(level);
    }
  }
}

function restartGame() {
  //reset game variables to initial state
  level = 1;
  wrongBallsEaten = goodBallsEaten = 0;
  currentScore = 0;
  numberOfLives = 5;

  startGame(level);

  gameState = "GameRunning";
}
function startGame(level) {
    globalSpeedMutiplier += 0.1;
    if (level < 5) nb = level;
    if (level>= 5) { 
        nb = level+2;
    }
  do {
    balls = createBalls(nb);
    initialNumberOfBalls = nb;
    numberOfGoodBalls = countNumberOfGoodBalls(balls, colorToEat);
  } while(numberOfGoodBalls === 0);
  
  wrongBallsEaten = goodBallsEaten = 0;
}

function countNumberOfGoodBalls(balls, colorToEat) {
  var nb = 0;
  
  balls.forEach(function(b) {
    if(b.color === colorToEat)
      nb++;
  });
  
  return nb;
}

function changeNbBalls(nb) {
  startGame(nb);
}

function changeColorToEat(color) {
  colorToEat = color;
}

function changePlayerColor(color) {
  player.color = color;
}

function changeBallSpeed(coef) {
    globalSpeedMutiplier = coef;
}

function mouseMoved(evt) {
    mousePos = getMousePos(canvas, evt);
}

function getMousePos(canvas, evt) {
    // necessary work in the canvas coordinate system
    var rect = canvas.getBoundingClientRect();
    return {
        x: evt.clientX - rect.left,
        y: evt.clientY - rect.top
    };
}

function movePlayerWithMouse() {
  if(mousePos !== undefined) {
    player.x = mousePos.x;
    player.y = mousePos.y;
  }
}

function mainLoop() {
  // 1 - clear the canvas
  ctx.clearRect(0, 0, w, h);

  switch(gameState) {
    case "StartMenu" :
      ctx.font = 'italic 20pt Calibri';
      ctx.fillText("Press <SPACE> to start the game", 20, 180+Math.random()*2);
      break;
    case "GameRunning": 
      updateGame();
      break;
    case "GameOverMenu" :
      ctx.font = 'italic 50pt Calibri';
      ctx.fillText("GAME OVER", 20, 100+Math.random()*20);
      ctx.fillText("Press <SPACE> to start a new game", 20, 180+Math.random()*20);
      break;
  }
  
  // ask the browser to call mainloop in 1/60 of  for a new animation frame
  requestAnimationFrame(mainLoop);
}

function updateGame() {
  // draw the ball and the player
  drawPlayer(player);
  drawAllBalls(balls);
  drawBallNumbers(balls);

  // animate the ball that is bouncing all over the walls
  moveAllBalls(balls);
  
  movePlayerWithMouse();
}
// Collisions between rectangle and circle
function circRectsOverlap(x0, y0, w0, h0, cx, cy, r) {
   var testX=cx;
   var testY=cy;
   if (testX < x0) testX=x0;
   if (testX > (x0+w0)) testX=(x0+w0);
   if (testY < y0) testY=y0;
   if (testY > (y0+h0)) testY=(y0+h0);
   return (((cx-testX)*(cx-testX)+(cy-testY)*(cy-testY))< r*r);
}

function createBalls(n) {
  // empty array
  var ballArray = [];
  
  // create n balls
  for(var i=0; i < n; i++) {
     var b = {
        x:w/2,
        y:h/2,
        radius: 5 + 30 * Math.random(), // between 5 and 35
        speedX: -5 + 10 * Math.random(), // between -5 and + 5
        speedY: -5 + 10 * Math.random(), // between -5 and + 5
        color:getARandomColor(),
      }
     // add ball b to the array
     ballArray.push(b);
    }
  // returns the array full of randomly created balls
  return ballArray;
}

function getARandomColor() {
  var colors = ['red', 'blue', 'cyan', 'purple', 'pink', 'green', 'yellow'];
  // a value between 0 and color.length-1
  // Math.round = rounded value
  // Math.random() a value between 0 and 1
  var colorIndex = Math.round((colors.length-1)*Math.random()); 
  var c = colors[colorIndex];
  
  // return the random color
  return c;
}

function drawBallNumbers(balls) { 
  ctx.save();
  ctx.font="20px Arial";
  
  if(balls.length === 0) {
    //ctx.fillText("Game Over!", 20, 30);
    level++;
    startGame(level);
  } else {
    ctx.fillText("Balls still alive: " + balls.length, 210, 30);
    ctx.fillText("Good Balls eaten: " + goodBallsEaten, 210, 50);
     ctx.fillText("Wrong Balls eaten: " + wrongBallsEaten, 210, 70);
     ctx.fillText("Lives left : " + numberOfLives, 210, 90);
     ctx.fillText("SCORE = " + currentScore, 210, 130);
  }
  ctx.restore();
}

function drawAllBalls(ballArray) {
    ballArray.forEach(function(b) {
      drawFilledCircle(b);
    });
}

function moveAllBalls(ballArray) {
  // iterate on all balls in array
  balls.forEach(function(b, index) {
      // b is the current ball in the array
      b.x += (b.speedX * globalSpeedMutiplier);
      b.y += (b.speedY * globalSpeedMutiplier);

      if(b.color === "green") 
        b.x += Math.random()*10;

  
      testCollisionBallWithWalls(b); 
    
      testCollisionWithPlayer(b, index);
  });
}

function testCollisionWithPlayer(b, index) {
  if(circRectsOverlap(player.x, player.y,
                     player.width, player.height,
                     b.x, b.y, b.radius)) {
    // we remove the element located at index
    // from the balls array
    // splice: first parameter = starting index
    //         second parameter = number of elements to remove
    if(b.color === colorToEat) {
      // Yes, we remove it and increment the score
      goodBallsEaten += 1;
      currentScore++;

      if(goodBallsEaten === numberOfGoodBalls) {
        level++;
        startGame(level);
      }
    } else {
      wrongBallsEaten += 1;

      // we decrement the number of lives
      numberOfLives--;

      if(numberOfLives === 0) {
        // display game over menu
        gameState = "GameOverMenu";
      }
    }
    
    balls.splice(index, 1);

  }
}

function testCollisionBallWithWalls(b) {
    // COLLISION WITH VERTICAL WALLS ?
    if((b.x + b.radius) > w) {
    // the ball hit the right wall
    // change horizontal direction
    b.speedX = -b.speedX;
    
    // put the ball at the collision point
    b.x = w - b.radius;
  } else if((b.x -b.radius) < 0) {
    // the ball hit the left wall
    // change horizontal direction
    b.speedX = -b.speedX;
    
    // put the ball at the collision point
    b.x = b.radius;
  }
  
  // COLLISIONS WTH HORIZONTAL WALLS ?
  // Not in the else as the ball can touch both
  // vertical and horizontal walls in corners
  if((b.y + b.radius) > h) {
    // the ball hit the right wall
    // change horizontal direction
    b.speedY = -b.speedY;
    
    // put the ball at the collision point
    b.y = h - b.radius;
  } else if((b.y -b.radius) < 0) {
    // the ball hit the left wall
    // change horizontal direction
    b.speedY = -b.speedY;
    
    // put the ball at the collision point
    b.Y = b.radius;
  }  
}

function drawPlayer(r) {
    // GOOD practice: save the context, use 2D trasnformations
    ctx.save();
  
    // translate the coordinate system, draw relative to it
    ctx.translate(r.x, r.y);
  
    ctx.fillStyle = r.color;
    // (0, 0) is the top left corner of the monster.
    ctx.fillRect(0, 0, r.width, r.height);
    // draw yellow eyes
    ctx.fillStyle = "yellow";
    //ctx.fillRect(4, 4, 3, 3);
    //ctx.fillRect(14, 4, 3, 3);
    ctx.beginPath();
    // centerX, centerY, radius, startAngle, endAngle
    ctx.arc(4, 4, 3, 0, Math.PI*2);
    ctx.fill();

    ctx.beginPath();
    // centerX, centerY, radius, startAngle, endAngle
    ctx.arc(14, 3, 3, 0, Math.PI*2);
    ctx.fill();
  
    // GOOD practice: restore the context
    ctx.restore();
}

function drawFilledCircle(c) {
    // GOOD practice: save the context, use 2D trasnformations
    ctx.save();
  
    // translate the coordinate system, draw relative to it
    ctx.translate(c.x, c.y);
  
    ctx.fillStyle = c.color;
    // (0, 0) is the top left corner of the monster.
    ctx.beginPath();
    ctx.arc(0, 0, c.radius, 0, 2*Math.PI);
    ctx.fill();
 
    // GOOD practice: restore the context
    ctx.restore();
}